@@ -1,5 +1,5 @@
 ;;; Mailing, forwarding, and replying commands for VM
-;;; Copyright (C) 1989-1997 Kyle E. Jones
+;;; Copyright (C) 1989-1998 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -44,13 +44,20 @@
 		     (let ((vm-summary-uninteresting-senders nil))
 		       (vm-sprintf 'vm-in-reply-to-format (car mp))))
 		in-reply-to (and (not (equal "" in-reply-to)) in-reply-to))
-	  (and subject vm-reply-subject-prefix
+	  (and subject (stringp vm-reply-subject-prefix)
 	       (let ((case-fold-search t))
-		 (not
-		  (equal
-		   (string-match (regexp-quote vm-reply-subject-prefix)
-				 subject)
-		   0)))
+		 (and
+		  (not
+		   (equal
+		    (string-match (regexp-quote vm-reply-subject-prefix)
+				  subject)
+		    0))
+		  (or
+		   (not (stringp vm-subject-ignored-prefix))
+		   (not
+		    (equal
+		     (string-match vm-subject-ignored-prefix subject)
+		     0)))))
 	       (setq subject (concat vm-reply-subject-prefix subject))))
 	 (t (cond ((setq tmp (vm-get-header-contents (car mp) "Reply-To:"
 						     ", "))
@@ -296,17 +303,18 @@ vm-included-text-prefix is prepended to every yanked line."
 		     (setq parts (copy-sequence (vm-mm-layout-parts o))))
 		    (t (setq parts (list o))))
 	      (while parts
-		(cond ((vm-mime-text-type-p (car parts))
+		(cond ((vm-mime-text-type-layout-p (car parts))
 		       (if (cond ((vm-mime-types-match
-				   "text/html"
-				   (car (vm-mm-layout-type (car parts))))
-				  (vm-mime-display-internal-text/html
-				   (car parts)))
-				 ((vm-mime-types-match
 				   "text/enriched"
 				   (car (vm-mm-layout-type (car parts))))
 				  (vm-mime-display-internal-text/enriched
 				   (car parts)))
+;; no text/html for now
+;;				 ((vm-mime-types-match
+;;				   "text/html"
+;;				   (car (vm-mm-layout-type (car parts))))
+;;				  (vm-mime-display-internal-text/html
+;;				   (car parts)))
 				 ((vm-mime-display-internal-text/plain
 				   (car parts) t)))
 			   nil
@@ -331,6 +339,11 @@ vm-included-text-prefix is prepended to every yanked line."
 			      (vm-text-end-of message))
 	    (setq end (vm-marker (+ start (- (vm-text-end-of message)
 					     (vm-headers-of message))) b)))))
+      ;; get rid of read-only text properties on the text, as
+      ;; they will only cause trouble.
+      (let ((inhibit-read-only t))
+	(remove-text-properties (point-min) (point-max) '(read-only nil)
+				(current-buffer)))
       (push-mark end)
       (cond (mail-citation-hook (run-hooks 'mail-citation-hook))
 	    (mail-yank-hooks (run-hooks 'mail-yank-hooks))
@@ -390,6 +403,84 @@ as having been replied to, if appropriate."
 	    (error "tale is an idiot, and so are you. :-)"))
 	(goto-char (vm-matched-header-end))))))
 
+(defun vm-mail-mode-insert-message-id-maybe ()
+  (if (not vm-mail-header-insert-message-id)
+      nil
+    (save-restriction
+      (save-excursion
+	(let ((resent nil)
+	      (time (current-time)))
+	  (if (or (vm-mail-mode-get-header-contents "Resent-To:")
+		  (vm-mail-mode-get-header-contents "Resent-Cc:")
+		  (vm-mail-mode-get-header-contents "Resent-Bcc:"))
+	      (progn
+		(vm-mail-mode-remove-header "Resent-Message-ID:")
+		(setq resent t))
+	    (vm-mail-mode-remove-header "Message-ID:"))
+	  (widen)
+	  (goto-char (point-min))
+	  (insert (format "%sMessage-ID: <%d.%d.%d.%d@%s>\n"
+			  (if resent "Resent-" "")
+			  (car time) (nth 1 time) (nth 2 time)
+			  (random 1000000)
+			  (system-name))))))))
+
+(defun vm-mail-mode-insert-date-maybe ()
+  (if (not vm-mail-header-insert-date)
+      nil
+    (save-restriction
+      (save-excursion
+	(let* ((timezone (car (current-time-zone)))
+	       (hour (/ timezone 3600))
+	       (min (/ (- timezone (* hour 3600)) 60))
+	       (time (current-time))
+	       (resent nil))
+	  (if (or (vm-mail-mode-get-header-contents "Resent-To:")
+		  (vm-mail-mode-get-header-contents "Resent-Cc:")
+		  (vm-mail-mode-get-header-contents "Resent-Bcc:"))
+	      (progn
+		(vm-mail-mode-remove-header "Resent-Date:")
+		(setq resent t))
+	    (vm-mail-mode-remove-header "Date:"))
+	  (widen)
+	  (goto-char (point-min))
+	  (insert (format "%sDate: " (if resent "Resent-" ""))
+		  (format-time-string "%a, " time)
+		  ;; %e generated " 2".  Go from string to int
+		  ;; to string to get rid of the blank.
+		  (int-to-string
+		   (string-to-int
+		    (format-time-string "%e" time)))
+		  (format-time-string " %b %Y %H:%M:%S" time)
+		  (format " %s%02d%02d"
+			  (if (< timezone 0) "-" "+")
+			  (abs hour)
+			  (abs min))
+		  (format-time-string " (%Z)" time)
+		  "\n"))))))
+
+(defun vm-mail-mode-remove-message-id-maybe ()
+  (if vm-mail-header-insert-message-id
+      (let ((resent nil))
+	(if (or (vm-mail-mode-get-header-contents "Resent-To:")
+		(vm-mail-mode-get-header-contents "Resent-Cc:")
+		(vm-mail-mode-get-header-contents "Resent-Bcc:"))
+	    (progn
+	      (vm-mail-mode-remove-header "Resent-Message-ID:")
+	      (setq resent t))
+	  (vm-mail-mode-remove-header "Message-ID:")))))
+
+(defun vm-mail-mode-remove-date-maybe ()
+  (if vm-mail-header-insert-date
+      (let ((resent nil))
+	(if (or (vm-mail-mode-get-header-contents "Resent-To:")
+		(vm-mail-mode-get-header-contents "Resent-Cc:")
+		(vm-mail-mode-get-header-contents "Resent-Bcc:"))
+	    (progn
+	      (vm-mail-mode-remove-header "Resent-Date:")
+	      (setq resent t))
+	  (vm-mail-mode-remove-header "Date:")))))
+
 (defun vm-mail-send ()
   "Just like mail-send except that VM flags the appropriate message(s)
 as replied to, forwarded, etc, if appropriate."
@@ -401,6 +492,8 @@ as replied to, forwarded, etc, if appropriate."
     (if (and vm-confirm-mail-send
 	     (not (y-or-n-p "Send the message? ")))
 	(error "Message not sent.")))
+  (vm-mail-mode-insert-date-maybe)
+  (vm-mail-mode-insert-message-id-maybe)
   ;; send mail using MIME if user requests it and if the buffer
   ;; has not already been MIME encoded.
   (if (and vm-send-using-mime
@@ -438,23 +531,22 @@ as replied to, forwarded, etc, if appropriate."
       (let ((this-command this-command))
 	(save-excursion
 	  (mail-send))))
-    (cond ((eq vm-system-state 'replying)
-	   (vm-mail-mark-replied))
-	  ((eq vm-system-state 'forwarding)
-	   (vm-mail-mark-forwarded))
-	  ((eq vm-system-state 'redistributing)
-	   (vm-mail-mark-redistributed)))
-    (vm-display nil nil '(vm-mail-send) '(vm-mail-send))
     ;; be careful, something could have killed the composition
     ;; buffer inside mail-send.
     (if (eq (current-buffer) composition-buffer)
 	(progn
+	  (cond ((eq vm-system-state 'replying)
+		 (vm-mail-mark-replied))
+		((eq vm-system-state 'forwarding)
+		 (vm-mail-mark-forwarded))
+		((eq vm-system-state 'redistributing)
+		 (vm-mail-mark-redistributed)))
 	  (vm-rename-current-mail-buffer)
-	  (vm-keep-mail-buffer (current-buffer))))))
+	  (vm-keep-mail-buffer (current-buffer))))
+    (vm-display nil nil '(vm-mail-send) '(vm-mail-send))))
 
 (defun vm-mail-mode-get-header-contents (header-name-regexp)
-  (let ((contents nil)
-	regexp)
+  (let (regexp)
     (setq regexp (concat "^\\(" header-name-regexp "\\)\\|\\(^"
 			 (regexp-quote mail-header-separator) "$\\)"))
     (save-excursion
@@ -469,6 +561,22 @@ as replied to, forwarded, etc, if appropriate."
 	      (vm-matched-header-contents)
 	    nil ))))))
 
+(defun vm-mail-mode-remove-header (header-name-regexp)
+  (let (regexp)
+    (setq regexp (concat "^\\(" header-name-regexp "\\)\\|\\(^"
+			 (regexp-quote mail-header-separator) "$\\)"))
+    (save-excursion
+      (save-restriction
+	(widen)
+	(goto-char (point-min))
+	(let ((case-fold-search t))
+	  (if (and (re-search-forward regexp nil t)
+		   (match-beginning 1)
+		   (progn (goto-char (match-beginning 0))
+			  (vm-match-header)))
+	      (delete-region (vm-matched-header-start) (vm-matched-header-end))
+	    nil ))))))
+
 (defun vm-rename-current-mail-buffer ()
   (if vm-rename-current-buffer-function
       (funcall vm-rename-current-buffer-function)
@@ -746,7 +854,10 @@ you can change the recipient address before resending the message."
 	(goto-char (point-min))
 	(if vm-mail-header-from
 	    (insert "Resent-From: " vm-mail-header-from ?\n))
-	(mail-position-on-field "Resent-To")
+	(if (vm-mail-mode-get-header-contents "Resent-To:")
+	    (mail-position-on-field "Resent-To")
+	  (insert "Resent-To: \n")
+	  (forward-char -1))
 	(setq default-directory dir)))
   (run-hooks 'vm-resend-bounced-message-hook)
   (run-hooks 'vm-mail-mode-hook))
@@ -946,7 +1057,8 @@ found, the current buffer remains selected."
 	  ;; different composition buffer.
 	  (vm-unbury-buffer b)
 	  (set-buffer b)
-	  (if (and vm-frame-per-composition (vm-multiple-frames-possible-p)
+	  (if (and vm-mutable-frames vm-frame-per-composition
+		   (vm-multiple-frames-possible-p)
 		   ;; only pop up a frame if there's an undisplay
 		   ;; hook in place to make the frame go away.
 		   vm-undisplay-buffer-hook)
@@ -1010,7 +1122,7 @@ found, the current buffer remains selected."
       (if (eq mail-aliases t)
 	  (progn
 	    (setq mail-aliases nil)
-	    (if (file-exists-p "~/.mailrc")
+	    (if (file-exists-p (or mail-personal-alias-file "~/.mailrc"))
 		(build-mail-aliases)))))
     (if (stringp vm-mail-header-from)
 	(insert "From: " vm-mail-header-from "\n"))
@@ -1021,11 +1133,11 @@ found, the current buffer remains selected."
     (and in-reply-to (insert "In-Reply-To: " in-reply-to "\n"))
     (and references (insert "References: " references "\n"))
     (insert "X-Mailer: VM " vm-version " under "
-	    (if vm-fsfemacs-19-p "Emacs " "")
+	    (if vm-fsfemacs-p "Emacs " "")
 	    emacs-version "\n")
     ;; REPLYTO environmental variable support
     ;; note that in FSF Emacs v19.29 we would initialize if the
-    ;; value was t.  nil is the treigger value used now.
+    ;; value was t.  nil is the trigger value used now.
     (and (eq mail-default-reply-to nil)
 	 (setq mail-default-reply-to (getenv "REPLYTO")))
     (if mail-default-reply-to
@@ -1044,7 +1156,7 @@ found, the current buffer remains selected."
 	     (insert mail-signature)))
 	  ((eq mail-signature t)
 	   (save-excursion
-	     (insert "-- \n")
+	     (insert "\n-- \n")
 	     (insert-file-contents (or (and (boundp 'mail-signature-file)
 					    (stringp mail-signature-file)
 					    mail-signature-file)
@@ -1053,7 +1165,8 @@ found, the current buffer remains selected."
     ;; config stuff will select it as the composition buffer.
     (vm-unbury-buffer (current-buffer))
     ;; make a new frame if the user wants it.
-    (if (and vm-frame-per-composition (vm-multiple-frames-possible-p))
+    (if (and vm-mutable-frames vm-frame-per-composition
+	     (vm-multiple-frames-possible-p))
 	(progn
 	  (vm-goto-new-frame 'composition)
 	  (vm-set-hooks-for-frame-deletion)))
@@ -1222,3 +1335,99 @@ found, the current buffer remains selected."
     (vm-send-mime-digest prefix))
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
+
+(defvar enriched-mode)
+
+(defun vm-preview-composition ()
+  "Show how the current composition buffer might be displayed
+in a MIME-aware mail reader.  VM copies and encodes the current
+mail composition buffer and displays it as a mail folder.
+Type `q' to quit this temp folder and return to composing your
+message."
+  (interactive)
+  (if (not (eq major-mode 'mail-mode))
+      (error "Command must be used in a VM Mail mode buffer."))
+  (let ((temp-buffer nil)
+	(mail-buffer (current-buffer))
+	(enriched (and (boundp 'enriched-mode) enriched-mode))
+	e-list)
+    (unwind-protect
+	(progn
+	  (setq temp-buffer (generate-new-buffer "composition preview"))
+	  (set-buffer temp-buffer)
+	  ;; so vm-mime-xxxx-encode-composition won't complain
+	  (setq major-mode 'mail-mode)
+	  (set (make-local-variable 'enriched-mode) enriched)
+	  (vm-insert-region-from-buffer mail-buffer)
+	  (goto-char (point-min))
+	  (or (vm-mail-mode-get-header-contents "From")
+	      (insert "From: " (user-login-name) "\n"))
+	  (or (vm-mail-mode-get-header-contents "Message-ID")
+	      (insert "Message-ID: <fake@fake.fake>\n"))
+	  (or (vm-mail-mode-get-header-contents "Date")
+	      (insert "Date: "
+		      (format-time-string "%a, %d %b %Y %H%M%S %Z"
+					  (current-time))
+		      "\n"))
+	  (and vm-send-using-mime
+	       (null (vm-mail-mode-get-header-contents "MIME-Version:"))
+	       (vm-mime-encode-composition))
+	  (vm-remove-mail-mode-header-separator)
+	  (goto-char (point-min))
+	  (insert (vm-leading-message-separator 'From_))
+	  (goto-char (point-max))
+	  (insert (vm-trailing-message-separator 'From_))
+	  (set-buffer-modified-p nil)
+	  ;; point of no return, don't kill it if the user quits
+	  (setq temp-buffer nil)
+	  (let ((vm-auto-decode-mime-messages t)
+		(vm-auto-displayed-mime-content-types t))
+	    (vm-save-buffer-excursion
+	     (vm-goto-new-folder-frame-maybe 'folder)
+	     (vm-mode)))
+	  (message
+	   (substitute-command-keys
+	    "Type \\[vm-quit] to continue composing your message"))
+	  ;; temp buffer, don't offer to save it.
+	  (setq buffer-offer-save nil)
+	  (vm-display (or vm-presentation-buffer (current-buffer)) t
+		      (list this-command) '(vm-mode startup)))
+      (and temp-buffer (kill-buffer temp-buffer)))))
+
+(defvar mail-send-actions)
+
+(defun vm-compose-mail (&optional to subject other-headers continue
+		        switch-function yank-action
+			send-actions)
+  (interactive)
+  (if continue
+      (vm-continue-composing-message)
+    (let ((buffer (vm-mail-internal
+		   (if to
+		       (format "message to %s" (vm-truncate-string to 20))
+		     nil)
+		   to subject)))
+      (re-search-forward (concat "^" mail-header-separator "$"))
+      (beginning-of-line)
+      (while other-headers
+	(insert (car (car other-headers)))
+	(while (eq (char-syntax (char-before (point))) ?\ )
+	  (delete-char -1))
+	(while (eq (char-before (point)) ?:)
+	  (delete-char -1))
+	(insert ": " (cdr (car other-headers)))
+	(if (not (eq (char-before (point)) ?\n))
+	    (insert "\n"))
+	(setq other-headers (cdr other-headers)))
+      (cond ((null to)
+	     (mail-position-on-field "To"))
+	    ((null subject)
+	     (mail-position-on-field "Subject"))
+	    (t
+	     (mail-text)))
+      (funcall (or switch-function (function switch-to-buffer))
+	       (current-buffer))
+      (if yank-action
+	  (funcall (car yank-action) (cdr yank-action)))
+      (make-local-variable 'mail-send-actions)
+      (setq mail-send-actions send-actions))))
