@@ -1,5 +1,5 @@
 ;;; Entry points for VM
-;;; Copyright (C) 1994-1997 Kyle E. Jones
+;;; Copyright (C) 1994-1998 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 (provide 'vm-startup)
 
+;;;###autoload
 (defun vm (&optional folder read-only)
   "Read mail under Emacs.
 Optional first arg FOLDER specifies the folder to visit.  It defaults
@@ -45,6 +46,7 @@ See the documentation for vm-mode for more information."
   ;; set enable-local-variables to nil for newer Emacses
   (catch 'done
     (let ((full-startup (not (bufferp folder)))
+	  (did-read-index-file nil)
 	  folder-buffer first-time totals-blurb
 	  preserve-auto-save-file)
       (setq folder-buffer
@@ -63,7 +65,7 @@ See the documentation for vm-mode for more information."
 			    (inhibit-local-variables t)
 			    (enable-local-variables nil)
 			    ;; for XEmacs/Mule
-			    (overriding-file-coding-system 'no-conversion))
+			    (coding-system-for-read 'no-conversion))
 			(message "Reading %s..." file)
 			(prog1 (find-file-noselect file)
 			  ;; update folder history
@@ -80,25 +82,25 @@ See the documentation for vm-mode for more information."
       ;; the file coding system and decoding it.
       ;; This is only possible if a file is visited and then vm-mode
       ;; is run on it afterwards.
-      (defvar file-coding-system)
+      (defvar buffer-file-coding-system)
       (if (and vm-xemacs-mule-p
-	       (not (eq (get-coding-system file-coding-system)
+	       (not (eq (get-coding-system buffer-file-coding-system)
 			(get-coding-system 'no-conversion-unix)))
-	       (not (eq (get-coding-system file-coding-system)
+	       (not (eq (get-coding-system buffer-file-coding-system)
 			(get-coding-system 'no-conversion-dos)))
-	       (not (eq (get-coding-system file-coding-system)
+	       (not (eq (get-coding-system buffer-file-coding-system)
 			(get-coding-system 'no-conversion-mac)))
-	       (not (eq (get-coding-system file-coding-system)
+	       (not (eq (get-coding-system buffer-file-coding-system)
 			(get-coding-system 'binary))))
 	  (let ((buffer-read-only nil)
 		(omodified (buffer-modified-p)))
 	    (unwind-protect
 		(progn
 		  (encode-coding-region (point-min) (point-max)
-					file-coding-system)
-		  (set-file-coding-system 'no-conversion nil)
+					buffer-file-coding-system)
+		  (set-buffer-file-coding-system 'no-conversion nil)
 		  (decode-coding-region (point-min) (point-max)
-					file-coding-system))
+					buffer-file-coding-system))
 	      (set-buffer-modified-p omodified))))
       (vm-check-for-killed-summary)
       (vm-check-for-killed-presentation)
@@ -129,23 +131,29 @@ See the documentation for vm-mode for more information."
       ;; needs to be done 
       (if first-time
 	  (progn
-	    (if (fboundp 'buffer-disable-undo)
-		(buffer-disable-undo (current-buffer))
-	      ;; obfuscation to make the v19 compiler not whine
-	      ;; about obsolete functions.
-	      (let ((x 'buffer-flush-undo))
-		(funcall x (current-buffer))))
+	    (buffer-disable-undo (current-buffer))
 	    (abbrev-mode 0)
 	    (auto-fill-mode 0)
-	    (vm-mode-internal)))
-      (vm-assimilate-new-messages nil t)
-      (if first-time
+	    (vm-mode-internal)
+	    ;; If the buffer is modified we don't know if the
+	    ;; folder format has been changed to be different
+	    ;; from index file, so don't read the index file in
+	    ;; that case.
+	    (if (not (buffer-modified-p))
+		(setq did-read-index-file (vm-read-index-file-maybe)))))
+
+      (vm-assimilate-new-messages nil (not did-read-index-file) nil)
+
+      (if (and first-time (not did-read-index-file))
 	  (progn
 	    (vm-gobble-visible-header-variables)
 	    (vm-gobble-bookmark)
+	    (vm-gobble-pop-retrieved)
 	    (vm-gobble-summary)
-	    (vm-gobble-labels)
-	    (vm-start-itimers-if-needed)))
+	    (vm-gobble-labels)))
+
+      (if first-time
+	  (vm-start-itimers-if-needed))
 
       ;; make a new frame if the user wants one.  reuse an
       ;; existing frame that is showing this folder.
@@ -158,11 +166,14 @@ See the documentation for vm-mode for more information."
       ;; raise frame if requested and apply startup window
       ;; configuration.
       (if full-startup
-	  (progn
+	  (let ((buffer-to-display (or vm-summary-buffer
+				       vm-presentation-buffer
+				       (current-buffer))))
+	    (vm-display buffer-to-display buffer-to-display
+			(list this-command)
+			(list (or this-command 'vm) 'startup))
 	    (if vm-raise-frame-at-startup
-		(vm-raise-frame))
-	    (vm-display nil nil (list this-command)
-			(list (or this-command 'vm) 'startup))))
+		(vm-raise-frame))))
 
       ;; say this NOW, before the non-previewers read a message,
       ;; alter the new message count and confuse themselves.
@@ -176,7 +187,9 @@ See the documentation for vm-mode for more information."
       ;; toolbar sets frame-specific height and width specifiers.
       (and (vm-toolbar-support-possible-p) vm-use-toolbar
 	   (progn
+	     (message "Initializing toolbar...")
 	     (vm-toolbar-install-toolbar)
+	     (message "Initializing toolbar... done")
 	     (vm-toolbar-update-toolbar)))
 
       (and vm-use-menus (vm-menu-support-possible-p)
@@ -193,6 +206,7 @@ See the documentation for vm-mode for more information."
 	    ;; raise the summary frame if the user wants frames
 	    ;; raised and if there is a summary frame.
 	    (if (and vm-summary-buffer
+		     vm-mutable-frames
 		     vm-frame-per-summary
 		     vm-raise-frame-at-startup)
 		(vm-raise-frame))
@@ -246,6 +260,7 @@ See the documentation for vm-mode for more information."
 	    (if (not (input-pending-p))
 		(message totals-blurb)))))))
 
+;;;###autoload
 (defun vm-other-frame (&optional folder read-only)
   "Like vm, but run in a newly created frame."
   (interactive (list nil current-prefix-arg))
@@ -260,6 +275,7 @@ See the documentation for vm-mode for more information."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-other-window (&optional folder read-only)
   "Like vm, but run in a different window."
   (interactive (list nil current-prefix-arg))
@@ -273,10 +289,11 @@ See the documentation for vm-mode for more information."
 
 (put 'vm-mode 'mode-class 'special)
 
+;;;###autoload
 (defun vm-mode (&optional read-only)
   "Major mode for reading mail.
 
-This is VM 6.28.
+This is VM 6.47.
 
 Commands:
    h - summarize folder contents
@@ -340,7 +357,7 @@ Commands:
 
  M N - use marks; the next vm command will affect only marked messages
        if it makes sense for the command to do so.  These commands
-       apply and remove marks to messages.
+       apply and remove marks to messages:
 
        M M - mark the current message
        M U - unmark the current message
@@ -356,6 +373,7 @@ Commands:
        M a - unmark messages with the same author as the current message
        M R - mark messages within the point/mark region in the summary
        M r - unmark messages within the point/mark region in the summary
+       M V - toggle the marked-ness of all messages
 
        M ? - partial help for mark commands
 
@@ -367,7 +385,11 @@ Commands:
  V V - visit a virtual folder (must be defined in vm-virtual-folder-alist)
  V C - create a virtual folder composed of a subset of the
        current folder's messages.
- V A - apply the selectors of a named virtual folder to the
+ V A - create a virtual folder containing all the messages in the current
+       folder with the same author as the current message.
+ V S - create a virtual folder containing all the messages in the current
+       folder with the same subject as the current message.
+ V X - apply the selectors of a named virtual folder to the
        messages in the current folder and create a virtual folder
        containing the selected messages.
  V M - toggle whether this virtual folder's messages mirror the
@@ -441,11 +463,13 @@ Variables:
    vm-frame-per-composition
    vm-frame-per-edit
    vm-frame-per-folder
+   vm-frame-per-help
    vm-frame-per-summary
    vm-highlighted-header-face
    vm-highlighted-header-regexp
    vm-honor-page-delimiters
    vm-image-directory
+   vm-index-file-suffix
    vm-in-reply-to-format
    vm-included-text-attribution-format
    vm-included-text-discard-header-regexp
@@ -458,9 +482,12 @@ Variables:
    vm-keep-sent-messages
    vm-mail-check-interval
    vm-mail-header-from
+   vm-mail-header-insert-date
+   vm-mail-header-insert-message-id
    vm-mail-mode-hook
    vm-make-crash-box-name
    vm-make-spool-file-name
+   vm-mime-7bit-composition-charset
    vm-mime-8bit-composition-charset
    vm-mime-8bit-text-transfer-encoding
    vm-mime-alternative-select-method
@@ -471,6 +498,7 @@ Variables:
    vm-mime-base64-decoder-switches
    vm-mime-base64-encoder-program
    vm-mime-base64-encoder-switches
+   vm-mime-button-format-alist
    vm-mime-button-face
    vm-mime-charset-font-alist
    vm-mime-default-face-charsets
@@ -492,7 +520,9 @@ Variables:
    vm-mutable-windows
    vm-netscape-program
    vm-netscape-program-switches
+   vm-pop-auto-expunge-alist
    vm-pop-bytes-per-session
+   vm-pop-expunge-after-retrieving
    vm-pop-max-message-size
    vm-pop-md5-program
    vm-pop-messages-per-session
@@ -539,6 +569,8 @@ Variables:
    vm-summary-thread-indent-level
    vm-tale-is-an-idiot
    vm-temp-file-directory
+   vm-thread-using-subject
+   vm-toolbar-pixmap-directory
    vm-trust-From_-with-Content-Length
    vm-undisplay-buffer-hook
    vm-unforwarded-header-regexp
@@ -558,6 +590,7 @@ Variables:
   (vm (current-buffer) read-only)
   (vm-display nil nil '(vm-mode) '(vm-mode)))
 
+;;;###autoload
 (defun vm-visit-folder (folder &optional read-only)
   "Visit a mail file.
 VM will parse and present its messages to you in the usual way.
@@ -596,6 +629,7 @@ visited folder."
     (setq folder (expand-file-name folder)))
   (vm folder read-only))
 
+;;;###autoload
 (defun vm-visit-folder-other-frame (folder &optional read-only)
   "Like vm-visit-folder, but run in a newly created frame."
   (interactive
@@ -624,6 +658,7 @@ visited folder."
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-visit-folder-other-window (folder &optional read-only)
   "Like vm-visit-folder, but run in a different window."
   (interactive
@@ -667,13 +702,16 @@ vm-visit-virtual-folder.")
 
 (defvar scroll-in-place)
 
+;;;###autoload
 (defun vm-visit-virtual-folder (folder-name &optional read-only)
   (interactive
    (let ((last-command last-command)
 	 (this-command this-command))
      (vm-session-initialization)
      (list
-      (vm-read-string "Visit virtual folder: " vm-virtual-folder-alist)
+      (vm-read-string (format "Visit%s virtual folder: "
+			      (if current-prefix-arg " read only" ""))
+		      vm-virtual-folder-alist)
       current-prefix-arg)))
   (vm-session-initialization)
   (if (not (assoc folder-name vm-virtual-folder-alist))
@@ -740,6 +778,7 @@ vm-visit-virtual-folder.")
 	  ;; raise the summary frame if the user wants frames
 	  ;; raised and if there is a summary frame.
 	  (if (and vm-summary-buffer
+		   vm-mutable-frames
 		   vm-frame-per-summary
 		   vm-raise-frame-at-startup)
 	      (vm-raise-frame))
@@ -761,6 +800,7 @@ vm-visit-virtual-folder.")
 	 (vm-display-startup-message)
 	 (message blurb))))
 
+;;;###autoload
 (defun vm-visit-virtual-folder-other-frame (folder-name &optional read-only)
   "Like vm-visit-virtual-folder, but run in a newly created frame."
   (interactive
@@ -768,8 +808,9 @@ vm-visit-virtual-folder.")
 	 (this-command this-command))
      (vm-session-initialization)
      (list
-      (vm-read-string "Visit virtual folder in other frame: "
-		       vm-virtual-folder-alist)
+      (vm-read-string (format "Visit%s virtual folder in other frame: "
+			      (if current-prefix-arg " read only" ""))
+		      vm-virtual-folder-alist)
       current-prefix-arg)))
   (vm-session-initialization)
   (if (vm-multiple-frames-possible-p)
@@ -780,6 +821,7 @@ vm-visit-virtual-folder.")
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
+;;;###autoload
 (defun vm-visit-virtual-folder-other-window (folder-name &optional read-only)
   "Like vm-visit-virtual-folder, but run in a different window."
   (interactive
@@ -787,8 +829,9 @@ vm-visit-virtual-folder.")
 	 (this-command this-command))
      (vm-session-initialization)
      (list
-      (vm-read-string "Visit virtual folder in other window: "
-		       vm-virtual-folder-alist)
+      (vm-read-string (format "Visit%s virtual folder in other window: "
+			      (if current-prefix-arg " read only" ""))
+		      vm-virtual-folder-alist)
       current-prefix-arg)))
   (vm-session-initialization)
   (if (one-window-p t)
@@ -798,30 +841,39 @@ vm-visit-virtual-folder.")
 	(vm-search-other-frames nil))
     (vm-visit-virtual-folder folder-name read-only)))
 
-(defun vm-mail ()
-  "Send a mail message from within VM, or from without."
+;;;###autoload
+(defun vm-mail (&optional to)
+  "Send a mail message from within VM, or from without.
+Optional argument TO is a string that should contain a comma separated
+recipient list."
   (interactive)
   (vm-session-initialization)
   (vm-select-folder-buffer)
   (vm-check-for-killed-summary)
-  (vm-mail-internal)
+  (vm-mail-internal nil to)
   (run-hooks 'vm-mail-hook)
   (run-hooks 'vm-mail-mode-hook))
 
-(defun vm-mail-other-frame ()
-  "Like vm-mail, but run in a newly created frame."
+;;;###autoload
+(defun vm-mail-other-frame (&optional to)
+  "Like vm-mail, but run in a newly created frame.
+Optional argument TO is a string that should contain a comma separated
+recipient list."
   (interactive)
   (vm-session-initialization)
   (if (vm-multiple-frames-possible-p)
       (vm-goto-new-frame 'composition))
   (let ((vm-frame-per-composition nil)
 	(vm-search-other-frames nil))
-    (vm-mail))
+    (vm-mail to))
   (if (vm-multiple-frames-possible-p)
       (vm-set-hooks-for-frame-deletion)))
 
-(defun vm-mail-other-window ()
-  "Like vm-mail, but run in a different window."
+;;;###autoload
+(defun vm-mail-other-window (&optional to)
+  "Like vm-mail, but run in a different window.
+Optional argument TO is a string that should contain a comma separated
+recipient list."
   (interactive)
   (vm-session-initialization)
   (if (one-window-p t)
@@ -829,8 +881,9 @@ vm-visit-virtual-folder.")
   (other-window 1)
   (let ((vm-frame-per-composition nil)
 	(vm-search-other-frames nil))
-    (vm-mail)))
+    (vm-mail to)))
 
+;;;###autoload
 (defun vm-submit-bug-report ()
   "Submit a bug report, with pertinent information to the VM bug list."
   (interactive)
@@ -887,6 +940,7 @@ vm-visit-virtual-folder.")
       'vm-frame-per-composition
       'vm-frame-per-edit
       'vm-frame-per-folder
+      'vm-frame-per-help
       'vm-frame-per-summary
       'vm-highlight-url-face
       'vm-highlighted-header-regexp
@@ -897,6 +951,7 @@ vm-visit-virtual-folder.")
       'vm-included-text-discard-header-regexp
       'vm-included-text-headers
       'vm-included-text-prefix
+      'vm-index-file-suffix
       'vm-init-file
       'vm-invisible-header-regexp
       'vm-jump-to-new-messages
@@ -904,11 +959,14 @@ vm-visit-virtual-folder.")
       'vm-keep-crash-boxes
       'vm-keep-sent-messages
       'vm-mail-header-from
+      'vm-mail-header-insert-date
+      'vm-mail-header-insert-message-id
       'vm-mail-hook
       'vm-make-crash-box-name
       'vm-make-spool-file-name
       'vm-mail-check-interval
       'vm-mail-mode-hook
+      'vm-mime-7bit-composition-charset
       'vm-mime-8bit-composition-charset
       'vm-mime-8bit-text-transfer-encoding
       'vm-mime-alternative-select-method
@@ -919,6 +977,7 @@ vm-visit-virtual-folder.")
       'vm-mime-base64-decoder-switches
       'vm-mime-base64-encoder-program
       'vm-mime-base64-encoder-switches
+      'vm-mime-button-format-alist
       'vm-mime-button-face
       'vm-mime-charset-font-alist
       'vm-mime-default-face-charsets
@@ -935,13 +994,17 @@ vm-visit-virtual-folder.")
       'vm-mosaic-program-switches
       'vm-move-after-deleting
       'vm-move-after-undeleting
+      'vm-move-after-killing
       'vm-move-messages-physically
       'vm-movemail-program
       'vm-mutable-frames
       'vm-mutable-windows
       'vm-netscape-program
       'vm-netscape-program-switches
+;; POP passwords might be listed here
+;;      'vm-pop-auto-expunge-alist
       'vm-pop-bytes-per-session
+      'vm-pop-expunge-after-retrieving
       'vm-pop-max-message-size
       'vm-pop-messages-per-session
       'vm-pop-md5-program
@@ -992,6 +1055,8 @@ vm-visit-virtual-folder.")
       'vm-summary-uninteresting-senders-arrow
       'vm-tale-is-an-idiot
       'vm-temp-file-directory
+      'vm-thread-using-subject
+      'vm-toolbar-pixmap-directory
       'vm-trust-From_-with-Content-Length
       'vm-undisplay-buffer-hook
       'vm-unforwarded-header-regexp
@@ -1033,12 +1098,14 @@ vm-visit-virtual-folder.")
 		       (< emacs-minor-version 14))))
 	 (error "VM %s must be run on XEmacs 19.14 or a later version."
 		vm-version))
-	((and vm-fsfemacs-19-p
+	((and vm-fsfemacs-p
 	      (or (< emacs-major-version 19)
 		  (and (= emacs-major-version 19)
 		       (< emacs-minor-version 34))))
-	 (error "VM %s must be run on Emacs 19.34 or a later version."
-		vm-version))))
+	 (error "VM %s must be run on Emacs 19.34 or a later v19 version."
+		vm-version))
+	((and vm-fsfemacs-p (= emacs-major-version 20))
+	 (error "VM has not been ported to v20 Emacs.  Running VM in this environment is not advised."))))
 
 (defun vm-set-debug-flags ()
   (or stack-trace-on-error
@@ -1050,12 +1117,14 @@ vm-visit-virtual-folder.")
 	      args-out-of-range
 	      void-function
 	      void-variable
+	      invalid-function
 	     ))))
 
 (defun vm-session-initialization ()
+  (require 'vm)
   (vm-note-emacs-version)
   (vm-check-emacs-version)
-  (vm-set-debug-flags)
+;;  (vm-set-debug-flags)
   ;; If this is the first time VM has been run in this Emacs session,
   ;; do some necessary preparations.
   (if (or (not (boundp 'vm-session-beginning))
@@ -1080,6 +1149,14 @@ vm-visit-virtual-folder.")
 	     (vm-menu-initialize-vm-mode-menu-map))
 	(setq vm-session-beginning nil))))
 
+(if (fboundp 'define-mail-user-agent)
+    (define-mail-user-agent 'vm-user-agent
+      (function vm-compose-mail)	; compose function
+      (function vm-send-mail-and-exit)	; send function
+      nil				; abort function (kill-buffer)
+      nil)				; hook variable (mail-send-hook)
+)
+
 (autoload 'reporter-submit-bug-report "reporter")
 (autoload 'timezone-make-date-sortable "timezone")
 (autoload 'rfc822-addresses "rfc822")
